@@ -1,6 +1,6 @@
 /*
   Mode switching tool for controlling flip flop (multiple device) USB gear
-  Version 1.1.0, 2010/01/22
+  Version 1.1.2, 2010/04/18
 
   Copyright (C) 2007, 2008, 2009, 2010 Josua Dietze (mail to "usb_admin" at the
   domain from the README; please do not post the complete address to the Net!
@@ -52,7 +52,7 @@
 
 /* Recommended tab size: 4 */
 
-char *version="1.1.0";
+char *version="1.1.2alpha";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -70,13 +70,16 @@ char *version="1.1.0";
 #define BUF_SIZE 4096
 #define DESCR_MAX 129
 
+#define SEARCH_DEFAULT 0
+#define SEARCH_TARGET 1
+
 #define SHOW_PROGRESS if (show_progress) printf
 
-int write_bulk(int endpoint, char *message, int length);
-int read_bulk(int endpoint, char *buffer, int length);
+//int write_bulk(int endpoint, char *message, int length);
+//int read_bulk(int endpoint, char *buffer, int length);
 
-int find_first_bulk_output_endpoint(struct usb_device *dev);
-int find_first_bulk_input_endpoint(struct usb_device *dev);
+//int find_first_bulk_output_endpoint(struct usb_device *dev);
+//int find_first_bulk_input_endpoint(struct usb_device *dev);
 
 char *TempPP=NULL;
 
@@ -91,11 +94,13 @@ int ret;
 
 char DetachStorageOnly=0, HuaweiMode=0, SierraMode=0, SonyMode=0, GCTMode=0;
 char verbose=0, show_progress=1, ResetUSB=0, CheckSuccess=0, config_read=0;
-char NeedResponse=0, InquireDevice=1, sysmode=0;
+char NeedResponse=0, NoDriverLoading=0, InquireDevice=1, sysmode=0;
 
 char imanufact[DESCR_MAX], iproduct[DESCR_MAX], iserial[DESCR_MAX];
 
 char MessageContent[LINE_DIM];
+char MessageContent2[LINE_DIM];
+char MessageContent3[LINE_DIM];
 char TargetProductList[LINE_DIM];
 char ByteString[LINE_DIM/2];
 char buffer[BUF_SIZE];
@@ -114,6 +119,8 @@ static struct option long_options[] = {
 	{"target-class",		required_argument, 0, 'C'},
 	{"message-endpoint",	required_argument, 0, 'm'},
 	{"message-content",		required_argument, 0, 'M'},
+	{"message-content2",	required_argument, 0, '2'},
+	{"message-content3",	required_argument, 0, '3'},
 	{"response-endpoint",	required_argument, 0, 'r'},
 	{"detach-only",			no_argument, 0, 'd'},
 	{"huawei-mode",			no_argument, 0, 'H'},
@@ -149,8 +156,11 @@ void readConfigFile(const char *configFilename)
 	ParseParamBool(configFilename, SierraMode);
 	ParseParamBool(configFilename, SonyMode);
 	ParseParamBool(configFilename, GCTMode);
+	ParseParamBool(configFilename, NoDriverLoading);
 	ParseParamHex(configFilename, MessageEndpoint);
 	ParseParamString(configFilename, MessageContent);
+	ParseParamString(configFilename, MessageContent2);
+	ParseParamString(configFilename, MessageContent3);
 	ParseParamHex(configFilename, NeedResponse);
 	ParseParamHex(configFilename, ResponseEndpoint);
 	ParseParamHex(configFilename, ResetUSB);
@@ -182,8 +192,6 @@ void printConfig()
 		printf ("TargetProduct=  not set\n");
 	if ( strlen(TargetProductList) )
 		printf ("TargetProductList=%s\n",		TargetProductList);
-//	else
-//		printf ("TargetProduct=  not set\n");
 	if ( TargetClass )
 		printf ("TargetClass=    0x%02x\n",		TargetClass);
 	else
@@ -201,6 +209,10 @@ void printConfig()
 		printf ("MessageContent=\"%s\"\n",	MessageContent);
 	else
 		printf ("MessageContent= not set\n");
+	if ( strlen(MessageContent2) )
+		printf ("MessageContent2=\"%s\"\n",	MessageContent2);
+	if ( strlen(MessageContent3) )
+		printf ("MessageContent3=\"%s\"\n",	MessageContent3);
 	printf ("NeedResponse=%i\n",		(int)NeedResponse);
 	if ( ResponseEndpoint )
 		printf ("ResponseEndpoint=0x%02x\n",	ResponseEndpoint);
@@ -234,9 +246,9 @@ int readArguments(int argc, char **argv)
 
 	while (1)
 	{
-		c = getopt_long (argc, argv, "heWQDndHSOGRIv:p:V:P:C:m:M:r:c:i:u:a:s:",
+		c = getopt_long (argc, argv, "heWQDndHSOGRIv:p:V:P:C:m:M:2:3:r:c:i:u:a:s:",
 						long_options, &option_index);
-	
+
 		/* Detect the end of the options. */
 		if (c == -1)
 			break;
@@ -251,6 +263,8 @@ int readArguments(int argc, char **argv)
 			case 'C': TargetClass = strtol(optarg, NULL, 16); break;
 			case 'm': MessageEndpoint = strtol(optarg, NULL, 16); break;
 			case 'M': strcpy(MessageContent, optarg); break;
+			case '2': strcpy(MessageContent2, optarg); break;
+			case '3': strcpy(MessageContent3, optarg); break;
 			case 'n': NeedResponse = 1; break;
 			case 'r': ResponseEndpoint = strtol(optarg, NULL, 16); break;
 			case 'd': DetachStorageOnly = 1; break;
@@ -268,7 +282,7 @@ int readArguments(int argc, char **argv)
 			case 'i': Interface = strtol(optarg, NULL, 16); break;
 			case 'u': Configuration = strtol(optarg, NULL, 16); break;
 			case 'a': AltSetting = strtol(optarg, NULL, 16); break;
-	
+
 			case 'e':
 				printVersion();
 				exit(0);
@@ -285,6 +299,7 @@ int readArguments(int argc, char **argv)
 				printf (" -C, --target-class NUM        target device class\n");
 				printf (" -m, --message-endpoint NUM    where to direct the message (optional)\n");
 				printf (" -M, --message-content <msg>   command to send (hex number as string)\n");
+				printf (" -2 <msg>, -3 <msg>            additional commands to send\n");
 				printf (" -n, --need-response           read a response to the message transfer\n");
 				printf (" -r, --response-endpoint NUM   where from read the response (optional)\n");
 				printf (" -d, --detach-only             just detach the storage driver\n");
@@ -304,7 +319,7 @@ int readArguments(int argc, char **argv)
 				printf (" -a, --altsetting NUM          select alternative USB interface setting\n\n");
 				exit(0);
 				break;
-		
+
 			default: //Unsupported - error message has already been printed
 				printf ("\n");
 				exit(1);
@@ -319,6 +334,7 @@ int main(int argc, char **argv)
 {
 	int numDefaults = 0, specialMode = 0, sonySuccess = 0;
 
+	signal(SIGTERM, release_usb_device);
 	/*
 	 * Parameter parsing, USB preparation/diagnosis, plausibility checks
 	 */
@@ -326,7 +342,7 @@ int main(int argc, char **argv)
 	// Check command arguments, use params instead of config file when given
 	switch (readArguments(argc, argv)) {
 		case 0:						// no argument or -W, -q or -s
-			readConfigFile("/etc/usb-modeswitch.conf");
+			readConfigFile("/etc/usb_modeswitch.setup");
 			break;
 		default:					// one or more arguments except -W, -q or -s 
 			if (!config_read)		// if arguments contain -c, the config file was already processed
@@ -372,7 +388,7 @@ int main(int argc, char **argv)
 	// Count existing target devices (remember for success check)
 	if (TargetVendor || TargetClass) {
 		SHOW_PROGRESS("Looking for target devices ...\n");
-		search_devices(&targetDeviceCount, TargetVendor, TargetProduct, TargetProductList, TargetClass);
+		search_devices(&targetDeviceCount, TargetVendor, TargetProduct, TargetProductList, TargetClass, SEARCH_TARGET);
 		if (targetDeviceCount) {
 			SHOW_PROGRESS(" Found devices in target mode or class (%d)\n", targetDeviceCount);
 		} else
@@ -381,17 +397,12 @@ int main(int argc, char **argv)
 
 	// Count default devices, return the last one found
 	SHOW_PROGRESS("Looking for default devices ...\n");
-	dev = search_devices(&numDefaults, DefaultVendor, DefaultProduct, "\0", TargetClass);
+	dev = search_devices(&numDefaults, DefaultVendor, DefaultProduct, "\0", TargetClass, SEARCH_DEFAULT);
 	if (numDefaults) {
-		SHOW_PROGRESS(" Found default devices (%d)\n", numDefaults);
-		if (TargetClass && !(TargetVendor || TargetProduct)) {
-			if ( dev != NULL ) {
-				SHOW_PROGRESS(" Found a default device NOT in target class mode\n");
-			} else {
-				SHOW_PROGRESS(" All devices in target class mode. Nothing to do. Bye.\n\n");
-				exit(0);
-			}
-		}
+		SHOW_PROGRESS(" Found devices in default mode or class (%d)\n", numDefaults);
+	} else {
+		SHOW_PROGRESS(" No devices in default mode or class found. Nothing to do. Bye.\n\n");
+		exit(0);
 	}
 	if (dev != NULL) {
 		devnum = dev->devnum;
@@ -416,7 +427,6 @@ int main(int argc, char **argv)
 
 	// Check or get endpoints if needed
 	if (!MessageEndpoint && (strlen(MessageContent) || InquireDevice) ) {
-//		SHOW_PROGRESS(" Finding endpoints ...\n");
 		MessageEndpoint = find_first_bulk_output_endpoint(dev);
 		if (!MessageEndpoint && strlen(MessageContent)) {
 			fprintf(stderr,"Error: message endpoint not given or found. Aborting.\n\n");
@@ -437,7 +447,7 @@ int main(int argc, char **argv)
 			SHOW_PROGRESS("Endpoints not found, skipping SCSI inquiry\n");
 			InquireDevice = 0;
 		}
-		
+
 	if (InquireDevice && show_progress) {
 		if (defaultClass == 0x08) {
 			SHOW_PROGRESS("Inquiring device details; driver will be detached ...\n");
@@ -465,7 +475,7 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	if ( !specialMode && !strlen(MessageContent) )
+	if ( !specialMode && !strlen(MessageContent) && AltSetting == -1 && Configuration == -1 )
 		SHOW_PROGRESS("Warning: no switching method given.\n");
 
 	/*
@@ -473,7 +483,7 @@ int main(int argc, char **argv)
 	 */
 
 	if (sysmode) {
-		openlog("usb-modeswitch", 0, LOG_SYSLOG);
+		openlog("usb_modeswitch", 0, LOG_SYSLOG);
 		syslog(LOG_NOTICE, "switching %04x:%04x (%s: %s)", DefaultVendor, DefaultProduct, imanufact, iproduct);
 	}
 
@@ -527,16 +537,16 @@ int main(int argc, char **argv)
 	}
 
 	if (CheckSuccess) {
-		signal(SIGTERM, release_usb_device);
 		if (checkSuccess()) {
-			if (sysmode)
-				printf("ok:%04x:%04x\n", TargetVendor, TargetProduct);
-			exit(0);
-		} else{
+			if (sysmode) {
+				if (NoDriverLoading)
+					printf("ok:\n");
+				else
+					printf("ok:%04x:%04x\n", TargetVendor, TargetProduct);
+			}
+		} else
 			if (sysmode)
 				printf("fail:\n");
-			exit(0);
-		}
 	} else {
 		if (SonyMode)
 			if (sonySuccess) {
@@ -552,15 +562,13 @@ int main(int argc, char **argv)
 			}
 		else
 			SHOW_PROGRESS("-> Run lsusb to note any changes. Bye.\n\n");
-		if (sysmode)
-			closelog();
-		exit(0);
 	}
 
+	if (sysmode)
+		closelog();
 	if (devh)
 		usb_close(devh);
-
-	return 0;
+	exit(0);
 }
 
 
@@ -703,7 +711,7 @@ void resetUSB ()
 
 int switchSendMessage ()
 {
-	int message_length, ret;
+	int ret;
 
 	SHOW_PROGRESS("Setting up communication with interface %d ...\n", Interface);
 	if (InquireDevice != 2) {
@@ -714,20 +722,20 @@ int switchSendMessage ()
 		}
 	}
 	usb_clear_halt(devh, MessageEndpoint);
-	SHOW_PROGRESS("Trying to send the message to endpoint 0x%02x ...\n", MessageEndpoint);
-	fflush(stdout);
+	SHOW_PROGRESS("Using endpoint 0x%02x for message sending ...\n", MessageEndpoint);
+	if (show_progress)
+		fflush(stdout);
 
-	message_length = strlen(MessageContent) / 2;
-	ret = write_bulk(MessageEndpoint, ByteString, message_length);
-	if (ret == -19)
+	if ( sendMessage(MessageContent, 1) )
 		goto skip;
 
-	if (NeedResponse) {
-		SHOW_PROGRESS("Reading the response to the message ...\n");
-		ret = read_bulk(ResponseEndpoint, ByteString, LINE_DIM/2);
-		if (ret == -19)
+	if (strlen(MessageContent2))
+		if ( sendMessage(MessageContent2, 2) )
+			goto skip;
+
+	if (strlen(MessageContent3))
+		if ( sendMessage(MessageContent3, 3) )
 			goto skip;
-	}
 
 	ret = usb_clear_halt(devh, MessageEndpoint);
 	if (ret)
@@ -844,19 +852,20 @@ int switchSonyMode ()
 		SHOW_PROGRESS(" OK, control message sent, waiting for device to return ...\n");
 
 	usb_close(devh);
+	devh = 0;
 
 	/* Now waiting for the device to reappear */
 	devnum=-1;
 	busnum=-1;
 	i=0;
-	dev = NULL;
-	while ( dev == NULL && i < 30 ) {
+	dev = 0;
+	while ( dev == 0 && i < 30 ) {
 		if ( i > 5 ) {
 			usb_find_busses();
 			usb_find_devices();
-			dev = search_devices(&found, DefaultVendor, DefaultProduct, NULL, TargetClass);
+			dev = search_devices(&found, DefaultVendor, DefaultProduct, "\0", TargetClass, SEARCH_TARGET);
 		}
-		if ( dev != NULL )
+		if ( dev != 0 )
 			break;
 		sleep(1);
 		if (show_progress) {
@@ -866,11 +875,10 @@ int switchSonyMode ()
 		i++;
 	}
 	SHOW_PROGRESS("\n After %d seconds:",i);
-	if ( dev != NULL ) {
+	if ( dev ) {
 		SHOW_PROGRESS(" device came back, proceeding\n");
-		devh = NULL;
 		devh = usb_open( dev );
-		if (devh == NULL) {
+		if (devh == 0) {
 			fprintf(stderr, "Error: could not get handle on device\n");
 			return 0;
 		}
@@ -881,9 +889,6 @@ int switchSonyMode ()
 
 	sleep(1);
 
-//	switchAltSetting();
-//	sleep(1);
-
 	SHOW_PROGRESS("Sending Sony control message again ...\n");
 	ret = usb_control_msg(devh, 0xc0, 0x11, 2, 0, buffer, 3, 100);
 	if (ret < 0) {
@@ -914,7 +919,6 @@ int detachDriver()
 	SHOW_PROGRESS(" OK, driver found (\"%s\")\n", buffer);
 	if (DetachStorageOnly && strcmp(buffer,"usb-storage")) {
 		SHOW_PROGRESS(" Warning: driver is not usb-storage\n");
-//		return 1;
 	}
 
 #ifndef LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP
@@ -926,22 +930,56 @@ int detachDriver()
 	ret = usb_detach_kernel_driver_np(devh, Interface);
 	if (ret == 0) {
 		SHOW_PROGRESS(" OK, driver \"%s\" detached\n", buffer);
-//		usb_clear_halt(devh, MessageEndpoint);
-//		usb_clear_halt(devh, ResponseEndpoint);
 	} else
 		SHOW_PROGRESS(" Driver \"%s\" detach failed with error %d. Trying to continue\n", buffer, ret);
 	return 1;
 }
 
 
+int sendMessage(char* message, int count)
+{
+	int message_length, ret;
+
+	if (strlen(message) % 2 != 0) {
+		fprintf(stderr, "Error: MessageContent %d hex string has uneven length. Skipping ...\n", count);
+		return 1;
+	}
+	message_length = strlen(message) / 2;
+	if ( hexstr2bin(message, ByteString, message_length) == -1) {
+		fprintf(stderr, "Error: MessageContent %d %s\n is not a hex string. Skipping ...\n", count, MessageContent);
+		return 1;
+	}
+	SHOW_PROGRESS("Trying to send message %d to endpoint 0x%02x ...\n", count, MessageEndpoint);
+	fflush(stdout);
+	ret = write_bulk(MessageEndpoint, ByteString, message_length);
+	if (ret == -19)
+		return 1;
+
+	if (NeedResponse) {
+		SHOW_PROGRESS("Reading the response to message %d ...\n", count);
+		ret = read_bulk(ResponseEndpoint, ByteString, 13);
+		if (ret == -19)
+			return 1;
+	}
+	return 0;
+}
+
+
 int checkSuccess()
 {
 	int i=0, ret;
 	int newTargetCount, success=0;
-	
+
 	SHOW_PROGRESS("\nChecking for mode switch (max. %d times, once per second) ...\n", CheckSuccess);
 	sleep(1);
 
+	// if target ID is not given but target class is, assign default as target;
+	// it will be needed for sysmode output
+	if (!TargetVendor && TargetClass) {
+		TargetVendor = DefaultVendor;
+		TargetProduct = DefaultProduct;
+	}
+
 	if (devh) // devh is 0 if device vanished during command transmission
 		for (i=0; i < CheckSuccess; i++) {
 
@@ -955,7 +993,7 @@ int checkSuccess()
 				if (i == CheckSuccess-1)
 					SHOW_PROGRESS(" If you want target checking, increase 'CheckSuccess' value.\n");
 				usb_close(devh);
-				devh = NULL;
+				devh = 0;
 				break;
 			} else
 				usb_release_interface(devh, Interface);
@@ -969,7 +1007,7 @@ int checkSuccess()
 		SHOW_PROGRESS(" Original device is gone already, not checking\n");
 
 
-	if ( (TargetVendor && (TargetProduct || strlen(TargetProductList))) || TargetClass )
+	if ( TargetVendor && (TargetProduct || strlen(TargetProductList)) )
 
 		// Recount target devices (compare with previous count) if target data is given.
 		// Target device on the same bus with higher device number is returned,
@@ -977,11 +1015,12 @@ int checkSuccess()
 		for (i=i; i < CheckSuccess; i++) {
 			SHOW_PROGRESS(" Searching for target devices ...\n");
 			usb_find_devices();
-			dev = search_devices(&newTargetCount, TargetVendor, TargetProduct, TargetProductList, TargetClass);
+			dev = search_devices(&newTargetCount, TargetVendor, TargetProduct, TargetProductList, TargetClass, SEARCH_TARGET);
 			if (dev && (newTargetCount > targetDeviceCount)) {
 				devh = usb_open(dev);
 				deviceDescription();
 				usb_close(devh);
+				devh = 0;
 				if (verbose) {
 					printf("\nFound target device %03d on bus %03d\n", \
 					dev->devnum, (int)strtol(dev->bus->dirname,NULL,10));
@@ -1062,8 +1101,10 @@ int read_bulk(int endpoint, char *buffer, int length)
 
 void release_usb_device(int dummy) {
 	SHOW_PROGRESS("Program cancelled by system. Bye.\n\n");
-	usb_release_interface(devh, Interface);
-	usb_close(devh);
+	if (devh) {
+		usb_release_interface(devh, Interface);
+		usb_close(devh);
+	}
 	if (sysmode)
 		closelog();
 	exit(0);
@@ -1073,13 +1114,13 @@ void release_usb_device(int dummy) {
 
 // iterates over busses and devices, counts the ones found and returns the last one of them
 
-struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList, int targetClass)
+struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList, int targetClass, int mode)
 {
 	struct usb_bus *bus;
 	char *listcopy, *token, buffer[2];
 	int devClass;
 	struct usb_device* right_dev = NULL;
-	
+
 	if ( targetClass && !(vendor || product) ) {
 		vendor = DefaultVendor;
 		product = DefaultProduct;
@@ -1138,26 +1179,51 @@ struct usb_device* search_devices( int *numFound, int vendor, int product, char*
 				if (product == dev->descriptor.idProduct) {
 					if (verbose)
 						printf ("   found matching product ID\n");
-					(*numFound)++;
-					devClass = dev->descriptor.bDeviceClass;
-					if (devClass == 0)
-						devClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
-					else
-						if (devClass != dev->config[0].interface[0].altsetting[0].bInterfaceClass)
+					if (targetClass == 0) {
+						(*numFound)++;
+						right_dev = dev;
+						if (verbose)
+							printf ("   adding device\n");
+					} else {
+						devClass = dev->descriptor.bDeviceClass;
+						if (devClass == 0)
 							devClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
-					if (busnum == -1) {
-						if (devClass != targetClass || targetClass == 0)
-							right_dev = dev;
-					} else
-						if (devClass == targetClass || targetClass == 0)
-							if (dev->devnum >= devnum && (int)strtol(dev->bus->dirname,NULL,10) == busnum)
+						else
+							// Check for some quirky devices
+							if (devClass != dev->config[0].interface[0].altsetting[0].bInterfaceClass)
+								devClass = dev->config[0].interface[0].altsetting[0].bInterfaceClass;
+						if (devClass == targetClass) {
+							if (verbose)
+								printf ("   target class %02x matching\n", targetClass);
+							if (mode == SEARCH_TARGET) {
+								(*numFound)++;
 								right_dev = dev;
+								if (verbose)
+									printf ("   adding device\n");
+							} else
+								if (verbose)
+									printf ("   not adding device as default\n");
+						} else {
+							if (verbose)
+								printf ("   target class %02x not matching\n", targetClass);
+							if (mode == SEARCH_DEFAULT) {
+								(*numFound)++;
+								right_dev = dev;
+								if (verbose)
+									printf ("   adding device as default\n");
+							}
+						}
+					}
+					// dirty hack: if busnum has other than init value, we're at
+					// success checking and probe for plausible new devnum/busnum
+					if (busnum != -1)
+						if (dev->devnum < devnum || (int)strtol(dev->bus->dirname,NULL,10) != busnum) {
+							if (verbose)
+								printf ("   busnum/devnum indicates an unrelated device\n");
+							right_dev = NULL;
+						}
 				}
-			if (right_dev && busnum != -1)
-				break;
 		}
-		if (right_dev && busnum != -1)
-			break;
 	}
 	if (productList != NULL)
 		free(listcopy);
@@ -1216,7 +1282,7 @@ char* ReadParseParam(const char* FileName, char *VariableName)
 	char *FirstQuote, *LastQuote, *P1, *P2;
 	int Line=0, Len=0, Pos=0;
 	FILE *file=fopen(FileName, "r");
-	
+
 	if (file==NULL) {
 		fprintf(stderr, "Error: Could not find file %s\n\n", FileName);
 		exit(1);
@@ -1245,7 +1311,7 @@ char* ReadParseParam(const char* FileName, char *VariableName)
 			*FirstQuote=*LastQuote='\0';
 			Equal=FirstQuote+1;
 		}
-		
+
 		// removes leading/trailing spaces
 		Pos=strspn (Str, " \t");
 		if (Pos==strlen(Str)) {
@@ -1271,7 +1337,7 @@ char* ReadParseParam(const char* FileName, char *VariableName)
 		}
 		Next:;
 	}
-	
+
 	// not found
 //	fprintf(stderr, "Error reading parameter file %s - Variable %s not found.", 
 //				FileName, VariableName);
@@ -1328,6 +1394,7 @@ void printVersion()
 {
 	printf("\n * usb-modeswitch: handle USB devices with multiple modes\n");
 	printf(" * Version %s (C) Josua Dietze 2010\n", version);
-	printf(" * Based on libusb 0.1.12\n\n");
+	printf(" * Based on libusb0 (0.1.12 and above)\n\n");
 	printf(" ! PLEASE REPORT NEW CONFIGURATIONS !\n\n");
 }
+
