@@ -2,7 +2,7 @@
   This file is part of usb-modeswitch, a mode switching tool for controlling
   flip flop (multiple device) USB gear
 
-  Version 1.1.0, 2010/01/19
+  Version 1.1.2, 2010/04/18
   Copyright (C) 2007, 2008, 2009, 2010  Josua Dietze
 
 
@@ -41,10 +41,11 @@ void switchAVMMode();
 int switchSonyMode();
 int detachDriver();
 int checkSuccess();
+int sendMessage(char* message, int count);
 int write_bulk(int endpoint, char *message, int length);
 int read_bulk(int endpoint, char *buffer, int length);
 void release_usb_device(int dummy);
-struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList, int targetClass);
+struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList, int targetClass, int mode);
 int find_first_bulk_output_endpoint(struct usb_device *dev);
 int find_first_bulk_input_endpoint(struct usb_device *dev);
 char* ReadParseParam(const char* FileName, char *VariableName);
@@ -56,7 +57,7 @@ int readArguments(int argc, char **argv);
 void deviceDescription();
 int deviceInquire();
 void resetUSB();
-
+void release_usb_device(int dummy);
 
 // Boolean
 #define  and     &&
@@ -99,9 +100,3 @@ extern char *TempPP;
 	if ((TempPP=ReadParseParam((ParamFileName), #B))!=NULL) \
 		B=(toupper(TempPP[0])=='Y' || toupper(TempPP[0])=='T'|| TempPP[0]=='1'); else B=0
 
-
-void release_usb_device(int dummy);
-
-struct usb_device* search_devices(int *numFound, int vendor, int product, char *productList, int targetClass);
-
-int hexstr2bin(const char *hex, char *buf, int len);
